
package org.hardknockjava.lesson09.exercise01;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Reader;
import java.io.Writer;
import java.util.Arrays;

public class Lesson09 {

    public static void main(String[] args) throws IOException {
        Lesson09 lesson = new Lesson09();
        String fileName = "lesson09-input.txt";

        String inputDirectoryName = "./input";
        File inputFile = new File(inputDirectoryName, fileName);

        System.out.println();
        System.out.println("The input file name is " + inputFile.getCanonicalPath());
        System.out.println();

        String outputDirectoryName = "./output";
        File outputDirectory = new File(outputDirectoryName);
        outputDirectory.mkdirs();

        try {
            lesson.displayFileUnbuffered(inputDirectoryName, inputFile, outputDirectory,
                    "lesson09-output-unbuffered.txt");
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    // This method uses an unbuffered
    // reader and writer. Notice how the number of characters
    // written to the writer is limited to the number of characters
    // read from the reader. If we don't do this, the entire
    // buffer is written even if it's not filled.
    // Also notice that any newline characters are copied
    // from input to output along with everything else.
    private void displayFileUnbuffered(String directoryName, File inputFile, File outputDirectory,
            String outputFileName) throws IOException {
        System.out.println("This is the unbuffered version.");

        File outputFile = new File(outputDirectory, outputFileName);
        deleteOutputFile(outputFile);

        {
            Reader inRdr = new FileReader(inputFile);
            Writer outWtr = new FileWriter(outputFile);

            char[] buffer = new char[1024];
            int charsRead;

            try {
                while ((charsRead = inRdr.read(buffer)) > 0) {
                    outWtr.write(buffer, 0, charsRead);
                    outWtr.flush();
                }
            } finally {
                inRdr.close();
                outWtr.close();
            }
        }

        {
            Reader outRdr = new FileReader(outputFile);
            char[] buffer = new char[1024];
            int charsRead;
            try {
                while ((charsRead = outRdr.read(buffer)) > 0) {
                    char[] lineChars = Arrays.copyOf(buffer, charsRead);
                    System.out.print(lineChars);
                }
            } finally {
                outRdr.close();
                System.out.println("\n\n******************************\n\n");
            }
        }
    }

    private void deleteOutputFile(File outputFile) {
        System.out.println();
        try {
            if (outputFile.delete()) {
                System.out.println("The output file " + outputFile.getCanonicalPath() + " was deleted.");
            } else {
                System.out.println("The output file " + outputFile.getCanonicalPath() + " was not deleted.");
            }
        } catch (IOException e) {
            System.out.println("An exception occurred when trying to delete the output file.");
        }
        System.out.println();
    }
}
